#!/usr/local/bin/perl

#- make use of procdir() recursive tree walker
require 'treewalk.pl';



##- batch file for pulling book info. 
## @echo off
## echo ISBN=%1 >>bookinfo.dat
## 
## REM -- use open ebook info if available?
## REM if exist %1.opf goto opf_info
## 
## :toc_info
## grep -i -q "./title" %1\%1_toc.htm >>bookinfo.dat
## REM grep -i -q "name=.author." %1\%1_toc.htm >>bookinfo.dat 
## grep -i -q "name=.Copyright." %1\%1_toc.htm >>bookinfo.dat 
## goto out
## 
## :opf_info
## grep -i -q "dc.Title" %1\%1_toc.htm >>bookinfo.dat
## REM grep -i -q "role=.aut." %1\%1_toc.htm >>bookinfo.dat 
## grep -i -q "dc.Rights" %1\%1_toc.htm >>bookinfo.dat 
## grep -i -q "dc.Date" %1\%1_toc.htm >>bookinfo.dat 
## 
## :out

##sub pull_opf_info {
##	local ($isbn) = @_;
##	local ($fname);
##
##	chdir $isbn;
##
##	$fname = sprintf("%s.opf", $isbn);
##
##	unless (open(FP, $fname)) {
##		print "error: Could not open $fname for reading\n";
##		return;
##	}
##
##	while (<FP>) {
##		if (m/role="aut"/) {
##			print $_;
##		}
##	}
##
##	close FP;
##
##	chdir "..";
##}

#<title>March to the Sea</title>
#<meta name="Copyright" content="&copy; 2001 by David Weber &amp; John Ringo" />
sub pull_toc_info {
	local ($file, $fullname) = @_;

#	print "pull_toc_info($file)\n";

	unless (open(FP, $fullname)) {
		print "error: Could not open $fullname for reading\n";
		return;
	}


	while (<FP>) {
		if (m;<title>(.*)</title>;i) { $title=$1; }
		if (m;<meta\s+name="copyright"\s+content="(.*)";i) 
			{ 
			$copyright=$1; 
			}
	} # end: for each line


	close FP;

}

#      <dc:Title>March to the Sea</dc:Title>
#      <dc:Creator role="aut" file-as="Weber, David ">David Weber</dc:Creator>
sub pull_opf_info {
	local ($file, $fullname) = @_;
	local ($a);

#	print "pull_opf_info($file)\n";

	unless (open(FP, $fullname)) {
		print "error: Could not open $fullname for reading\n";
		return;
	}


	$a = 0;
	while (<FP>) {
		if (m;<dc:Title>(.*)</dc:Title>;i) { $title=$1; }
		if (m;<dc:Rights>(.*)</dc:Rights>;i) { $copyright=$1; }
		if (m;<dc:Creator\s+role="aut"\s+file-as="(.*)">(.*)</dc:Creator>;i) 
			{ 
			$aut_last[$a] = $1;			# Ringo, John
			$aut[$a] = $2;			# John Ringo
			$a++;
			}
	} # end: for each line


	close FP;

}

sub procfile {
		local ($path, $file) = @_;
		local ($fullname);
		local ($fullurl);
		local ($i);
		local ($is_opf);

		$fullname = $path . "/" . $file;
		# print "file=$file, path=$path, fullname=$fullname\n";


		$isbn = "no isbn";
		$title = "no title";
		$copyright = "";		# from toc file
		@aut = ();				# from opf file: John Ringo
		@aut_last = ();			# from opf file: Ringo, John

		if ($file =~ m;(.*)_toc.htm;) { $isbn=$1; }
		if ($file =~ m;(.*).opf;) { $isbn=$1; }

		if (defined $indexed{$isbn}) {
			return;
		}

		print "file=$file, path=$path, fullname=$fullname\n";

		if ($file =~ m/\.opf$/) {
			pull_opf_info($file, $fullname);
			$is_opf = 1;
		} else {
			pull_toc_info($file, $fullname);
			$is_opf = 0;
		}

#	print "copyright len=", (length $copyright), " aut count=", $#aut, "\n";
#- $#aut == -1 if empty

	# we have to create the copyright notice from the authors array
	if (length $copyright == 0) {
		$copyright = "Copyright";
		$sep = " ";
		foreach $name (@aut) {
			$copyright = sprintf("%s%s%s", $copyright, $sep, $name);
			$sep = ", ";
		}
	}
	$fullurl = sprintf("%s/%s_toc.htm", $isbn, $isbn);

	##!! need to convert ampersand to HTML entity
	##	i.e. & --> &amp;
	## note: some already _have_ the proper &amp; so
	## if we blindly convert all ampersands we mess things
	## up, therefore convert only ampersands followed by spaces
	##

	$copyright =~ s/\& /\&amp; /;
	$title =~ s/\& /\&amp; /;

	$indexed{$isbn} = $title;

	print OFP <<EOT;
<tr><td bgcolor="#eeeeee">
<a href="$fullurl">$title</a>
</td><td bgcolor="#ddeedd">
EOT

	if ($is_opf == 1) {
#		foreach $name (@aut) {
#			print OFP "$name<br>\n";
#		}
		print OFP "$copyright\n";
	} else {
		print OFP "$copyright\n";
	}

	print OFP "</td></tr>\n";

#	print OFP <<EOT;
#</td></tr>
#EOT

	## for XML need to remove some entities
	##	i.e. &copy; --> Copyright
	$copyright =~ s/\&copy;/Copyright/;


	print XFP <<EOT;
<BOOK>
<ISBN>$isbn</ISBN>
<BOOKTITLE>$title</BOOKTITLE>
<COPYRIGHT>$copyright</COPYRIGHT>
EOT


	if ($#aut_last < 0) {
		print XFP "<AUTHOR>(see copyright)</AUTHOR>\n";
	} else {
		$n = 1;
		foreach $name (@aut_last) {
			print XFP "<AUTHOR>$name</AUTHOR>\n";
	#		print "--> aut=$name\n";
			$n++;
		}
	}

	print XFP "</BOOK>\n";

}


unless (open(OFP, ">bookindex.htm")) {
	print "error: Could not open bookindex.htm for writing\n";
	exit;
}


unless (open(XFP, ">bookinfo.xml")) {
	print "error: Could not open bookinfo.xml for writing\n";
	close OFP;
	exit;
}

print OFP <<EOT;
<html>
<head>
<title>Baen Book Index</title>
</head>
<body>
<h1>Baen Book Index</h1>
<table border=3>
<tr><th>Title</th><th>Copyright</th></tr>
EOT

print XFP <<EOT;
<?xml version='1.0'?>

<?xml-stylesheet type="text/xsl" href="bookinfo.xslt"?>

<!--
<?xml:stylesheet type="text/xsl" href="bookinfo.xsl"?>
<?xml-stylesheet type="text/xsl" href="bookinfo.xsl"?>
-->

<LIBRARY>
EOT


@indexed = ();

#	do procdir('.', "\\.opf\$|toc\\.htm\$");
	do procdir('.', "\\.opf\$");
	do procdir('.', "toc\\.htm\$");



	print OFP <<EOT;
</table>
</body>
</html>
EOT

	print XFP <<EOT;
</LIBRARY>
EOT


	close OFP;
	close XFP;

